package ch.zhaw.pm2.racetrack.strategy;

import ch.zhaw.pm2.racetrack.PositionVector;
import ch.zhaw.pm2.racetrack.Track;

import java.util.*;

public class PathFinderMoveStrategy implements MoveStrategy{
    private Track track;
    private int carIndex;
    private List<PositionVector.Direction> moveList;
    private int pointer;
    private List<PositionVector.Direction> allDirections;


    public PathFinderMoveStrategy(Track track, int carIndex) {
        this.track = track;
        this.carIndex = carIndex;
        allDirections = Arrays.asList(PositionVector.Direction.values());
        createMoveList();
    }


    private void createMoveList(){
        PossibleMove finishedMove = null;
        List<PossibleMove> possibleMoves= new ArrayList<>();
        for(PositionVector.Direction direction : allDirections){
            possibleMoves.add(new PossibleMove(null, direction));
        }
        while(finishedMove == null){
            List<PossibleMove> newMoves = new ArrayList<>();
            for(PossibleMove previousMove : possibleMoves){
                for(PositionVector.Direction direction : allDirections){
                    PossibleMove newMove = new PossibleMove(previousMove, direction);

                        if(! (newMove.crashed() || newMove.drivingLoop())){
                            newMoves.add(newMove);
                        }
                }
            }
            possibleMoves = newMoves;
            finishedMove = findFinishedMove(possibleMoves);
        }

        moveList = finishedMove.directions;

        pointer = moveList.size();


    }

    private PossibleMove findFinishedMove(List<PossibleMove> moveList){
        for(PossibleMove move : moveList){
            if(move.finished()){
                return move;
            }
        }
        return null;
    }


    public class PossibleMove {
        List<PositionVector.Direction> directions;
        List<PositionVector> positions;
        PositionVector endVelocity;

        public PossibleMove(PossibleMove previousMove, PositionVector.Direction nextDirection){
            PositionVector startVelocity;
            PositionVector startPosition;
            positions = new ArrayList<>();
            directions = new ArrayList<>();
            if(previousMove != null){
                positions.addAll(previousMove.positions);
                directions.addAll(previousMove.directions);
                startPosition = positions.get(positions.size()-1);
                startVelocity = previousMove.endVelocity;
            }
            else {
                startPosition = track.getCarPos(carIndex);
                startVelocity = track.getCar(carIndex).getVelocity();
            }
            directions.add(nextDirection);
            endVelocity = new PositionVector(startVelocity.getX() + nextDirection.vector.getX(), startVelocity.getY() + nextDirection.vector.getY());
            positions.add(new PositionVector(startPosition.getX() + endVelocity.getX(), startPosition.getY() + endVelocity.getY()));
        }

        public boolean finished(){
            if(track.calculateNewWinPoints(positions.get(positions.size()-2), positions.get(positions.size()-1)) == 1){
                return true;
            }
            else return false;
        }

        public boolean crashed() {
                for(PositionVector point : track.calculatePointsOnPath(positions.get(positions.size()-1), positions.get(positions.size()-2))) {
                    if (track.willCrashAtPosition(carIndex, point)) {
                        return true;
                    }
                }
            return false;
        }

        public boolean drivingLoop(){
            for(int i = 0; i < positions.size()-1; i++){
                if(positions.get(i).equals(positions.get(positions.size()-1))){
                    return true;
                }
            }
            return false;
        }

    }



    @Override
    public PositionVector.Direction nextMove() {
        pointer -= 1;
        //TODO: Check if crash. if yes --> createMoveList();
        if (pointer >= 0) {
            return moveList.get(pointer);
        }
        return null;
    }


}
